2.4 Duration with Envelopes

# Duração com Envelopes

Numa secção anterior vimos como podemos usar o comando `sleep` para controlar quando tocar os nossos sons. Mas ainda não sabemos como controlar a duração dos nossos sons.

Para termos um meio simples, mas eficaz, de controlar a duração dos nossos sons, o Sonic Pi tem a noção de *envelope de amplitude ADSR* (vamos falar sobre o que ADSR significa mais à frente nesta secção). Um envelope de amplitude oferece dois aspectos úteis de controlo:

controlo sobre a duração de um som
controlo sobre a amplitude de um som

## Duração

Duração é o tempo que o som dura. Duração mais longa significa que vais ouvir o som durante mais tempo. Todos os sons no Sonic Pi têm um envelope de amplitude controlável, e a duração total desse envelope é a duração do som. Portanto, controlando o envelope, contrlas a duração.

## Amplitude

Além de controlar a duração, o envelope ADSR também oferece um *controlo excelente sobre a amplitude do som*. Todos os sons audíveis começam e acabam com silêncio e contêm uma parte não silenciosa no meio. Envelopes permitem deslizar e mantêr a amplitude das partes não silenciosas do som. É como dar instruções de como aumentar e diminuir o volume de um amplificador de guitarra. Por exemplo, podes pedir a alguém para "começar com silêncio, aumentar o volume lentamente até ao máximo, manter durante um bocado e depois diminuir rapidamente até silêncio". O Sonic Pi permite programar este comportamento com envelopes.

Para recapitular, como vimos antes, uma amplitude de 0 é silêncio e uma amplitude de 1 é volume normal.

Agora, vamos ver cada uma das partes dos envelopes.

## Fase de Relaxamento

A única parte do envelope que é usada por defeito é o tempo de repouso (ou relaxamento). Isto é o tempo que o som de um synth leva até silenciar. Todos os synths têm, por defeito, um tempo de repouso de 1, o que significa que duram uma batida (que é 1 segundo, para BPM de 60):

```
play 70
```

Esta nota vai-se ouvir por 1 segundo. Força, podes contar :-) Isto é a versão curta da versão mais longa e explicita:

```
play 70, release: 1
```

Nota como soam exactamente iguais (o som dura 1 segundo). No entanto, agora é muito fácil alterar a duração mudando o valor da opt `release:`:

```
play 60, release: 2
```

Podemos fazer com que o som dure muito pouco tempo usando um valor muito baixo para `release:`:

```
play 60, release: 0.2
```

A duração do repouso do som chama-se *fase de relaxamento* (ou repouso) e por defeito é uma transição linear (isto é, uma linha recta). O diagrama seguinte ilustra esta transição:

![release envelope](../images/tutorial/env-release.png)

A linha vertical à esquerda do diagrama mostra que o som começa com amplitude 0, mas vai até amplitude máxima imediatamente (isto é a fase de ataque, que vamos dar a seguir). Uma vez a amplitude máxima, esta volta a baixar numa linha recta até 0, levando o tempo especificado pela opt `release:`. *Tempos de repouso mais longos produzem fade outs mais longos.*

Pode então mudar a duração do teu som ao mudar o tempo de repouso. Experimenta mudar os tempos de repouso da tua música.

## Fase de Ataque

Por defeito, a *fase de ataque* é 0 para todos os synths, o que significa que a amplitude sobe de 0 para 1 imediatamente. Isto dá um som percurssivo inicial. No entanto, podes querer que o teu som tenha um *fade in*. Isto pode ser feito com a opt `attack:`. Experimenta usar fade in em alguns sons:

```
play 60, attack: 2
sleep 3
play 65, attack: 0.5
```

Pode querer usar váras opts ao mesmo tempo. Por exemplo, para um ataque curto e repouso longo, experimenta:

```
play 60, attack: 0.7, release: 4
```

Esta ataque curto e repouso longo está ilustrado no diagrama seguinte:

![attack release envelope](../images/tutorial/env-attack-release.png)

Claro que podes trocar as coisas. Tenta um ataque longo e repouso curto:

```
play 60, attack: 4, release: 0.7
```

![long attack short release envelope](../images/tutorial/env-long-attack-short-release.png)

Finalmente, também podes ter tanto o tempo de ataque como de pouso curtos para sons curtos.

```
play 60, attack: 0.5, release: 0.5
```

![short attack short release envelope](../images/tutorial/env-short-attack-short-release.png)

## Fase de Sustentação

Além de especificar os tempos de ataque e repouso, também podes mudar o tempo de sustentação para controlar a *fase de sustentação*. Isto é o tempo a que a amplitude se vai manter no máximo, entre as fases de ataque e repouso.

```
play 60, attack: 0.3, sustain: 1, release: 1
```

![ASR envelope](../images/tutorial/env-attack-sustain-release.png)

O tempo de sustentação é útil para quando queres dar a notar sons importantes na música, antes de entrar na fase de repouso. Claro, é válido dar o valor de `0` às opts `attack:` e `release:` e usar só a sustentação para o som não ter nenhum fade in ou fade out. No entanto, estás avisado, um tempo de repouso de 0 pode produzir *clicks* no som e é melhor usar um valor muito baixo como `0.2`.

## Fase de Decaimento

Para um nível extra de controlo, também podes especificar um tempo de decaimento. Esta é a fase do envelope que fica entre as fases de ataque e sustentação e especifica o tempo em que a amplitude vai baixar do `attack_level:` para o `decay_level:` (que, a não ser que o mudes explicitamente, vai ser o mesmo que o `sustain_level:`). Por defeito, a opt `decay:` é 0 e tanto o ataque (`attack_leve:`) como a sustentação (`sustain_level:`) são 1, portanto precisas de especificá-los para o tempo de decaimento fazer efeito:

```
play 60, attack: 0.1, attack_level: 1, decay: 0.2, sustain_level: 0.4, sustain: 1, release: 0.5
```

![ADSR envelope](../images/tutorial/env-attack-decay-sustain-release.png)

## Nível de Decaimento

Um último truque é que apesar da opt `decay_level:`, por defeito, ser igual ao `sustain_level:` podes dar-lhes valores diferentes para controlo total sobre o envelope. Isto permite-te criar envelopes como o seguinte:

```
play 60, attack: 0.1, attack_level: 1, decay: 0.2, decay_level: 0.3, sustain: 1, sustain_level: 0.4, release: 0.5
```

![ASR envelope](../images/tutorial/env-decay-level.png)

Também é possível `decay_level:` ter um valor superior ao do `sustain_level`

```
play 60, attack: 0.1, attack_level: 0.1, decay: 0.2, decay_level: 1, sustain: 0.5, sustain_level: 0.8, release: 1.5
```

![ASR envelope](../images/tutorial/env-decay-level-2.png)

## Envelopes ADSR

Em resumo, envelopes ADSR no Sonic Pi têm as seguintes fases:

*ataque* - tempo de amplitude 0 até ao `attack_level:`
*decaimento* - tempo para mudar a amplitude de `attack_level:` até `sustain_level:`
*sustentação* - tempo para mudar a amplitude de `decay_level:` até `sustain_level:`
*repouso* - tempo para mudar a amplitude de `sustain_leve:` até 0

É importante notar que a duração de um som é a soma dos tempos de cada uma das fases. Portanto, o som seguinte vai durar 0.5 + 1 + 2 + 0.5 = 4 batidas:

```
play 60, attack: 0.5, attack_level: 1, decay: 1, sustain_level: 0.4, sustain: 2, release: 0.5
```

Agora vai experimentar usar envelopes aos teus sons...
